@@ -4,7 +4,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -21,22 +21,17 @@
 #define AFS_ENV			1
 #define AFS_SUN_ENV		1
 #define AFS_SUN5_ENV		1
-#define	AFS_SUN52_ENV		1
-#define	AFS_SUN53_ENV		1
-#define	AFS_SUN54_ENV		1
-#define	AFS_SUN55_ENV		1
-#define	AFS_SUN56_ENV		1
-#define AFS_SUN57_ENV		1
-#define AFS_SUN58_ENV		1
 #define AFS_SUN59_ENV		1
 #define AFS_SUN510_ENV		1
+#define AFS_SUN511_ENV		1
 #define AFS_X86_ENV		1
 
-#define AFS_64BIT_ENV		1	/* Defines afs_int32 as int, not long. */
-#define AFS_64BIT_CLIENT	1	
+#define AFS_64BIT_CLIENT	1
 
 #define AFS_HAVE_FLOCK_SYSID    1
 
+#define AFS_CACHE_VNODE_PATH 1
+
 #include <afs/afs_sysnames.h>
 
 #define AFS_GLOBAL_SUNLOCK	1	/* For global locking */
@@ -49,19 +44,21 @@
 #define	AFS_3DISPARES		1	/* Utilize the 3 available disk inode 'spares' */
 #endif /* AFS_NAMEI_ENV */
 
-#define	AFS_SYSCALL		65
+/* so we get _IOW() when we include sys/ioctl.h */
+#define BSD_COMP
 
 /* File system entry (used if mount.h doesn't define MOUNT_AFS */
 #define AFS_MOUNT_AFS		"afs"
 
 /* Machine / Operating system information */
-#define sys_sunx86_510		1
-#define SYS_NAME		"sunx86_510"
-#define SYS_NAME_ID		SYS_NAME_ID_sunx86_510
+#define sys_sunx86_511		1
+#define SYS_NAME		"sunx86_511"
+#define SYS_NAME_ID		SYS_NAME_ID_sunx86_511
 #define AFSLITTLE_ENDIAN	1
 #define AFS_HAVE_FFS		1	/* Use system's ffs. */
 #define AFS_HAVE_VXFS		1	/* Support cache on Veritas vxfs file system */
 #define AFS_HAVE_STATVFS	1	/* System supports statvfs */
+#define AFS_HAVE_STATVFS64	1	/* System supports statvfs64 */
 #define AFS_VM_RDWR_ENV		1	/* read/write implemented via VM */
 #define AFS_USE_GETTIMEOFDAY	1	/* use gettimeofday to implement rx clock */
 
@@ -79,10 +76,9 @@
 /* sun definitions here */
 #define	AFS_UIOFMODE		1	/* Only in afs/afs_vnodeops.c (afs_ustrategy) */
 #define	AFS_SYSVLOCK		1	/* sys v locking supported */
-/*#define	AFS_USEBUFFERS	1*/
 #define	afsio_iov		uio_iov
 #define	afsio_iovcnt		uio_iovcnt
-#define	afsio_offset		uio_offset
+#define	afsio_offset		uio_loffset
 #define	afsio_seg		uio_segflg
 #define	afsio_fmode		uio_fmode
 #define	afsio_resid		uio_resid
@@ -90,36 +86,21 @@
 #define	AFS_UIOUSER		UIO_USERSPACE
 #define	AFS_CLBYTES		MCLBYTES
 #define	AFS_MINCHANGE		2
-#define	osi_GetTime(x)		uniqtime(x)
+#define	osi_GetTime(x)		uniqtime32(x)
 
 /**
   * These defines are for the 64 bit Solaris 7 port
   * AFS_SYSCALL32 is used to protect the ILP32 syscall interface
-  * AFS_64BIT_ENV is for use of 64 bit inode numbers
   */
-#if defined(__sparcv9)
-#define	AFS_SUN57_64BIT_ENV	1
+#if defined(__amd64)
+#define	AFS_SUN5_64BIT_ENV	1
 #define AFS_64BIT_INO   	1
 #endif
 
-/**
-  * Solaris 7 64 bit has two versions of uniqtime. Since we consistently
-  * use 32 bit quantities for time in afs, we now use uniqtime32
-  */
-#if defined(AFS_SUN57_64BIT_ENV)
-#undef osi_GetTime
-#define osi_GetTime(x)		uniqtime32(x)
-#endif
-
-
-
 #define	AFS_KALLOC(n)		kmem_alloc(n, KM_SLEEP)
 #define AFS_KALLOC_NOSLEEP(n)   kmem_alloc(n, KM_NOSLEEP)
 #define	AFS_KFREE		kmem_free
 #define	VATTR_NULL		vattr_null
-#define memset(A, B, S)		bzero(A, S)
-#define memcpy(B, A, S)		bcopy(A, B, S)
-#define memcmp(A, B, S)		bcmp(A, B, S)
 #endif /* KERNEL */
 #define	AFS_DIRENT
 #ifndef CMSERVERPREF
@@ -151,9 +132,6 @@
 #define AFS_USR_SUN8_ENV        1
 #define AFS_USR_SUN9_ENV        1
 
-#define AFS_64BIT_ENV           1
-
-
 #include <afs/afs_sysnames.h>
 
 #if 0
@@ -161,24 +139,26 @@
 #endif
 
 #define	AFS_3DISPARES		1	/* Utilize the 3 available disk inode 'spares' */
-#define	AFS_SYSCALL		65
+
+/* so we get _IOW() when we include sys/ioctl.h */
+#define BSD_COMP
 
 /* File system entry (used if mount.h doesn't define MOUNT_AFS */
 #define AFS_MOUNT_AFS		1
 
 /* Machine / Operating system information */
-#define sys_sunx86_510          1
-#define SYS_NAME                "sunx86_510"
-#define SYS_NAME_ID             SYS_NAME_ID_sunx86_510
+#define sys_sunx86_511         1
+#define SYS_NAME                "sunx86_511"
+#define SYS_NAME_ID             SYS_NAME_ID_sunx86_511
 #define AFSLITTLE_ENDIAN	1
 #define AFS_HAVE_FFS            1	/* Use system's ffs. */
-#define AFS_HAVE_STATVFS	0	/* System doesn't support statvfs */
+#define AFS_HAVE_STATVFS	1	/* System supports statvfs */
+#define AFS_HAVE_STATVFS64	1	/* System supports statvfs64 */
 
 /* Extra kernel definitions (from kdefs file) */
 #ifdef KERNEL
 #define	AFS_UIOFMODE		1	/* Only in afs/afs_vnodeops.c (afs_ustrategy) */
 #define	AFS_SYSVLOCK		1	/* sys v locking supported */
-/*#define	AFS_USEBUFFERS	1*/
 #define	afsio_iov		uio_iov
 #define	afsio_iovcnt		uio_iovcnt
 #define	afsio_offset		uio_offset
