@@ -16,12 +16,16 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /cvs/openafs/src/libadmin/samples/cm_local_cell.c,v 1.4.4.1 2003/10/24 05:24:51 shadow Exp $");
+#include <roken.h>
 
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #include <pthread.h>
 #endif
+
+#include <rx/rx.h>
+#include <rx/rxstat.h>
+
 #include <afs/afs_Admin.h>
 #include <afs/afs_clientAdmin.h>
 #include <afs/afs_utilAdmin.h>
@@ -32,18 +36,15 @@ pthread_mutex_t des_random_mutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t rxkad_random_mutex = PTHREAD_MUTEX_INITIALIZER;
 #endif /* AFS_DARWIN_ENV */
 
-void Usage()
+void
+Usage(void)
 {
-    fprintf(stderr,
-	    "Usage: cm_local_cell <host> <port>\n");
+    fprintf(stderr, "Usage: cm_local_cell <host> <port>\n");
     exit(1);
 }
 
-void ParseArgs(
-    int argc,
-    char *argv[],
-    char **srvrName,
-    long *srvrPort)
+void
+ParseArgs(int argc, char *argv[], char **srvrName, long *srvrPort)
 {
     char **argp = argv;
 
@@ -59,7 +60,8 @@ void ParseArgs(
 	Usage();
 }
 
-int main(int argc, char *argv[])
+int
+main(int argc, char *argv[])
 {
     int rc;
     afs_status_t st = 0;
@@ -108,7 +110,8 @@ int main(int argc, char *argv[])
     }
 
     printf("\n");
-    printf("Client %s (port %d) is in cell %s\n", srvrName, srvrPort, cellName);
+    printf("Client %s (port %ld) is in cell %s\n", srvrName, srvrPort,
+	   cellName);
     printf("\n");
 
     exit(0);
