@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -10,42 +10,39 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-
 #ifndef KERNEL
-#ifndef AFS_NT40_ENV
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-#include <net/if.h>
-#include <netinet/in.h>
-#include <sys/ioctl.h>
-#include <string.h>
-#if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
-#include <sys/sysctl.h>
-#include <net/route.h>
-#include <net/if_dl.h>
-#endif
+
+# include <roken.h>
+# ifndef AFS_NT40_ENV
+# include <net/if.h>
+#  if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#   include <sys/sysctl.h>
+#   ifndef AFS_ARM_DARWIN_ENV
+#    include <net/route.h>
+#   endif
+#   include <net/if_dl.h>
+#  endif
 
 /*
  * By including this, we get any system dependencies. In particular,
  * the pthreads for solaris requires the socket call to be mapped.
  */
-#include "rx.h"
-#include "rx_globals.h"
-#endif /* AFS_NT40_ENV */
+#  include "rx.h"
+#  include "rx_globals.h"
+# endif /* AFS_NT40_ENV */
 #else /* KERNEL */
-#ifdef UKERNEL
-#include "rx/rx_kcommon.h"
-#else /* UKERNEL */
-#include "rx/rx.h"
-#endif /* UKERNEL */
+# ifdef UKERNEL
+#  include "rx/rx_kcommon.h"
+# else /* UKERNEL */
+#  include "rx/rx.h"
+# endif /* UKERNEL */
 #endif /* KERNEL */
 
 #define NIFS		512
 
 #if defined(AFS_USR_DFBSD_ENV)
-#include <net/if.h>
-#include <sys/sockio.h>
+# include <net/if.h>
+# include <sys/sockio.h>
 #endif
 
 #ifdef KERNEL
@@ -137,11 +134,49 @@ rt_xaddrs(caddr_t cp, caddr_t cplim, struct rt_addrinfo *rtinfo)
 }
 #endif
 
+static_inline int
+rxi_IsLoopbackIface(struct sockaddr_in *a, unsigned long flags)
+{
+    afs_uint32 addr = ntohl(a->sin_addr.s_addr);
+    if (rx_IsLoopbackAddr(addr)) {
+	return 1;
+    }
+    if ((flags & IFF_LOOPBACK) && ((addr & 0xff000000) == 0x7f000000)) {
+	return 1;
+    }
+    return 0;
+}
 
-/* this function returns the total number of interface addresses 
+/* this function returns the total number of interface addresses
 ** the buffer has to be passed in by the caller
 */
 #if defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
+#if defined(AFS_OBSD42_ENV)
+void
+ifm_fixversion(char *buffer, size_t *size) {
+    struct if_msghdr *ifm;
+    char *b = buffer;
+    char *s, *t;
+
+    if ((t = malloc(*size)) != NULL) {
+	memcpy(t, buffer, *size);
+
+	for (s = t; s < t + *size; s += ifm->ifm_msglen) {
+	    ifm = (struct if_msghdr *)s;
+
+	    if (ifm->ifm_version == RTM_VERSION) {
+		memcpy(b, ifm, ifm->ifm_msglen);
+		b += ifm->ifm_msglen;
+	    }
+	}
+
+	free(t);
+
+	*size = b - buffer;
+    }
+}
+#endif
+
 int
 rx_getAllAddr_internal(afs_uint32 buffer[], int maxSize, int loopbacks)
 {
@@ -168,6 +203,9 @@ rx_getAllAddr_internal(afs_uint32 buffer[], int maxSize, int loopbacks)
 	free(buf);
 	return 0;
     }
+#if defined(AFS_OBSD42_ENV)
+    ifm_fixversion(buf, &needed);
+#endif
     lim = buf + needed;
     next = buf;
     while (next < lim) {
@@ -209,7 +247,7 @@ rx_getAllAddr_internal(afs_uint32 buffer[], int maxSize, int loopbacks)
 	    if (count >= maxSize)	/* no more space */
 		dpf(("Too many interfaces..ignoring 0x%x\n",
 		       a->sin_addr.s_addr));
-	    else if (!loopbacks && a->sin_addr.s_addr == htonl(0x7f000001)) {
+	    else if (!loopbacks && rxi_IsLoopbackIface(a, ifm->ifm_flags)) {
 		addrcount--;
 		continue;	/* skip loopback address as well. */
 	    } else if (loopbacks && ifm->ifm_flags & IFF_LOOPBACK) {
@@ -254,6 +292,9 @@ rx_getAllAddrMaskMtu(afs_uint32 addrBuffer[], afs_uint32 maskBuffer[],
 	free(buf);
 	return 0;
     }
+#if defined(AFS_OBSD42_ENV)
+    ifm_fixversion(buf, &needed);
+#endif
     s = socket(PF_INET, SOCK_DGRAM, 0);
     if (s < 0)
 	return 0;
@@ -295,13 +336,13 @@ rx_getAllAddrMaskMtu(afs_uint32 addrBuffer[], afs_uint32 maskBuffer[],
 	    }
 	    a = (struct sockaddr_in *) info.rti_info[RTAX_IFA];
 
-	    if (a->sin_addr.s_addr != htonl(0x7f000001) ) {
+	    if (!rx_IsLoopbackAddr(ntohl(a->sin_addr.s_addr))) {
 		if (count >= maxSize) {	/* no more space */
 		    dpf(("Too many interfaces..ignoring 0x%x\n",
 			   a->sin_addr.s_addr));
 		} else {
 		    struct ifreq ifr;
-		    
+
 		    addrBuffer[count] = a->sin_addr.s_addr;
 		    a = (struct sockaddr_in *) info.rti_info[RTAX_NETMASK];
 		    if (a)
@@ -374,9 +415,9 @@ rx_getAllAddr_internal(afs_uint32 buffer[], int maxSize, int loopbacks)
 	 cp += sizeof(ifr->ifr_name) + MAX(a->sin_len, sizeof(*a))
 #endif
 #endif
-	) 
+	)
 #else
-    for (i = 0; i < len; ++i) 
+    for (i = 0; i < len; ++i)
 #endif
     {
 #if    defined(AFS_AIX41_ENV) || defined (AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
@@ -396,10 +437,10 @@ rx_getAllAddr_internal(afs_uint32 buffer[], int maxSize, int loopbacks)
 	}
 	if (a->sin_addr.s_addr != 0) {
             if (!loopbacks) {
-                if (a->sin_addr.s_addr == htonl(0x7f000001)) 
+                if (rxi_IsLoopbackIface(a, ifr->ifr_flags))
 		    continue;	/* skip loopback address as well. */
             } else {
-                if (ifr->ifr_flags & IFF_LOOPBACK) 
+                if (ifr->ifr_flags & IFF_LOOPBACK)
 		    continue;	/* skip aliased loopbacks as well. */
 	    }
 	    if (count >= maxSize)	/* no more space */
@@ -483,7 +524,7 @@ rx_getAllAddrMaskMtu(afs_uint32 addrBuffer[], afs_uint32 maskBuffer[],
 		continue;	/* ignore this address */
 	    }
 
-            if (a->sin_addr.s_addr == htonl(0x7f000001) )
+            if (rx_IsLoopbackAddr(ntohl(a->sin_addr.s_addr)))
                 continue;   /* skip loopback address as well. */
 
 	    if (count >= maxSize) {	/* no more space */
